@@ -14,9 +14,12 @@
 
 import mock
 
+from six import StringIO
 
 from zun.common import consts
 from zun.common import exception
+from zun.compute import claims
+from zun.compute import compute_node_tracker
 from zun.compute import manager
 import zun.conf
 from zun.objects.container import Container
@@ -26,6 +29,12 @@ from zun.tests.unit.container.fake_driver import FakeDriver as fake_driver
 from zun.tests.unit.db import utils
 
 
+class FakeResourceTracker(object):
+
+    def container_claim(self, context, container, host, limits):
+        return claims.NopClaim()
+
+
 class TestManager(base.TestCase):
 
     def setUp(self):
@@ -47,77 +56,78 @@ class TestManager(base.TestCase):
     @mock.patch.object(Container, 'save')
     @mock.patch('zun.image.driver.pull_image')
     @mock.patch.object(fake_driver, 'create')
-    @mock.patch.object(fake_driver, 'create_sandbox')
-    def test_container_create(self, mock_create_sandbox, mock_create,
-                              mock_pull, mock_save):
+    def test_container_create(self, mock_create, mock_pull, mock_save):
         container = Container(self.context, **utils.get_test_container())
         image = {'image': 'repo', 'path': 'out_path', 'driver': 'glance'}
         mock_pull.return_value = image, False
-        mock_create_sandbox.return_value = 'fake_id'
-        self.compute_manager._do_container_create(self.context, container)
+        self.compute_manager._resource_tracker = FakeResourceTracker()
+        networks = []
+        self.compute_manager._do_container_create(self.context, container,
+                                                  networks)
         mock_save.assert_called_with(self.context)
         mock_pull.assert_any_call(self.context, container.image, 'latest',
                                   'always', 'glance')
-        mock_create.assert_called_once_with(self.context, container,
-                                            'fake_id', image)
+        mock_create.assert_called_once_with(self.context, container, image,
+                                            networks)
 
     @mock.patch.object(Container, 'save')
-    @mock.patch.object(fake_driver, 'create_sandbox')
     @mock.patch('zun.image.driver.pull_image')
     @mock.patch.object(manager.Manager, '_fail_container')
     def test_container_create_pull_image_failed_docker_error(
-            self, mock_fail, mock_pull, mock_create_sandbox, mock_save):
+            self, mock_fail, mock_pull, mock_save):
         container = Container(self.context, **utils.get_test_container())
         mock_pull.side_effect = exception.DockerError("Pull Failed")
-        mock_create_sandbox.return_value = mock.MagicMock()
-        self.compute_manager._do_container_create(self.context, container)
+        networks = []
+        self.compute_manager._do_container_create(self.context, container,
+                                                  networks)
         mock_fail.assert_called_once_with(self.context,
                                           container, "Pull Failed")
 
     @mock.patch.object(Container, 'save')
-    @mock.patch.object(fake_driver, 'create_sandbox')
     @mock.patch('zun.image.driver.pull_image')
     @mock.patch.object(manager.Manager, '_fail_container')
     def test_container_create_pull_image_failed_image_not_found(
-            self, mock_fail, mock_pull, mock_create_sandbox, mock_save):
+            self, mock_fail, mock_pull, mock_save):
         container = Container(self.context, **utils.get_test_container())
         mock_pull.side_effect = exception.ImageNotFound("Image Not Found")
-        mock_create_sandbox.return_value = mock.MagicMock()
-        self.compute_manager._do_container_create(self.context, container)
+        networks = []
+        self.compute_manager._do_container_create(self.context, container,
+                                                  networks)
         mock_fail.assert_called_once_with(self.context,
                                           container, "Image Not Found")
 
     @mock.patch.object(Container, 'save')
-    @mock.patch.object(fake_driver, 'create_sandbox')
     @mock.patch('zun.image.driver.pull_image')
     @mock.patch.object(manager.Manager, '_fail_container')
     def test_container_create_pull_image_failed_zun_exception(
-            self, mock_fail, mock_pull, mock_create_sandbox, mock_save):
+            self, mock_fail, mock_pull, mock_save):
         container = Container(self.context, **utils.get_test_container())
         mock_pull.side_effect = exception.ZunException(
             message="Image Not Found")
-        mock_create_sandbox.return_value = mock.MagicMock()
-        self.compute_manager._do_container_create(self.context, container)
+        networks = []
+        self.compute_manager._do_container_create(self.context, container,
+                                                  networks)
         mock_fail.assert_called_once_with(self.context,
                                           container, "Image Not Found")
 
     @mock.patch.object(Container, 'save')
     @mock.patch('zun.image.driver.pull_image')
     @mock.patch.object(fake_driver, 'create')
-    @mock.patch.object(fake_driver, 'create_sandbox')
     @mock.patch.object(manager.Manager, '_fail_container')
     def test_container_create_docker_create_failed(self, mock_fail,
-                                                   mock_create_sandbox,
                                                    mock_create, mock_pull,
                                                    mock_save):
         container = Container(self.context, **utils.get_test_container())
-        image = {'image': 'repo', 'path': 'out_path', 'driver': 'glance'}
+        image = {'image': 'repo', 'path': 'out_path', 'driver': 'glance',
+                 'repo': 'test', 'tag': 'testtag'}
         mock_pull.return_value = image, False
         mock_create.side_effect = exception.DockerError("Creation Failed")
-        mock_create_sandbox.return_value = mock.MagicMock()
-        self.compute_manager._do_container_create(self.context, container)
-        mock_fail.assert_called_once_with(self.context,
-                                          container, "Creation Failed")
+        self.compute_manager._resource_tracker = FakeResourceTracker()
+        networks = []
+        self.compute_manager._do_container_create(self.context, container,
+                                                  networks)
+        mock_fail.assert_called_once_with(
+            self.context, container, "Creation Failed", unset_host=True)
 
     @mock.patch.object(Container, 'save')
     @mock.patch('zun.image.driver.pull_image')
@@ -130,61 +140,79 @@ class TestManager(base.TestCase):
         mock_create.return_value = container
         mock_pull.return_value = image, False
         container.status = 'Stopped'
-        self.compute_manager._do_container_run(self.context, container)
+        self.compute_manager._resource_tracker = FakeResourceTracker()
+        networks = []
+        self.compute_manager._do_container_run(self.context, container,
+                                               networks)
         mock_save.assert_called_with(self.context)
         mock_pull.assert_any_call(self.context, container.image, 'latest',
                                   'always', 'glance')
-        mock_create.assert_called_once_with(self.context, container,
-                                            None, image)
-        mock_start.assert_called_once_with(container)
+        mock_create.assert_called_once_with(self.context, container, image,
+                                            networks)
+        mock_start.assert_called_once_with(self.context, container)
 
     @mock.patch.object(Container, 'save')
     @mock.patch('zun.image.driver.pull_image')
     @mock.patch.object(manager.Manager, '_fail_container')
     def test_container_run_image_not_found(self, mock_fail,
                                            mock_pull, mock_save):
-        container = Container(self.context, **utils.get_test_container())
+        container_dict = utils.get_test_container(
+            image='test:latest', image_driver='docker',
+            image_pull_policy='ifnotpresent')
+        container = Container(self.context, **container_dict)
         mock_pull.side_effect = exception.ImageNotFound(
             message="Image Not Found")
+        networks = []
         self.compute_manager._do_container_run(self.context,
-                                               container)
+                                               container,
+                                               networks)
         mock_save.assert_called_with(self.context)
         mock_fail.assert_called_with(self.context,
                                      container, 'Image Not Found')
-        mock_pull.assert_called_once_with(self.context, 'kubernetes/pause',
-                                          'latest', 'ifnotpresent', 'docker')
+        mock_pull.assert_called_once_with(self.context, 'test', 'latest',
+                                          'ifnotpresent', 'docker')
 
     @mock.patch.object(Container, 'save')
     @mock.patch('zun.image.driver.pull_image')
     @mock.patch.object(manager.Manager, '_fail_container')
     def test_container_run_image_pull_exception_raised(self, mock_fail,
                                                        mock_pull, mock_save):
-        container = Container(self.context, **utils.get_test_container())
+        container_dict = utils.get_test_container(
+            image='test:latest', image_driver='docker',
+            image_pull_policy='ifnotpresent')
+        container = Container(self.context, **container_dict)
         mock_pull.side_effect = exception.ZunException(
             message="Image Not Found")
+        networks = []
         self.compute_manager._do_container_run(self.context,
-                                               container)
+                                               container,
+                                               networks)
         mock_save.assert_called_with(self.context)
         mock_fail.assert_called_with(self.context,
                                      container, 'Image Not Found')
-        mock_pull.assert_called_once_with(self.context, 'kubernetes/pause',
-                                          'latest', 'ifnotpresent', 'docker')
+        mock_pull.assert_called_once_with(self.context, 'test', 'latest',
+                                          'ifnotpresent', 'docker')
 
     @mock.patch.object(Container, 'save')
     @mock.patch('zun.image.driver.pull_image')
     @mock.patch.object(manager.Manager, '_fail_container')
     def test_container_run_image_pull_docker_error(self, mock_fail,
                                                    mock_pull, mock_save):
-        container = Container(self.context, **utils.get_test_container())
+        container_dict = utils.get_test_container(
+            image='test:latest', image_driver='docker',
+            image_pull_policy='ifnotpresent')
+        container = Container(self.context, **container_dict)
         mock_pull.side_effect = exception.DockerError(
             message="Docker Error occurred")
+        networks = []
         self.compute_manager._do_container_run(self.context,
-                                               container)
+                                               container,
+                                               networks)
         mock_save.assert_called_with(self.context)
         mock_fail.assert_called_with(self.context,
                                      container, 'Docker Error occurred')
-        mock_pull.assert_called_once_with(self.context, 'kubernetes/pause',
-                                          'latest', 'ifnotpresent', 'docker')
+        mock_pull.assert_called_once_with(self.context, 'test', 'latest',
+                                          'ifnotpresent', 'docker')
 
     @mock.patch.object(Container, 'save')
     @mock.patch('zun.image.driver.pull_image')
@@ -194,26 +222,38 @@ class TestManager(base.TestCase):
                                                       mock_fail,
                                                       mock_pull, mock_save):
         container = Container(self.context, **utils.get_test_container())
-        mock_pull.return_value = {'name': 'nginx', 'path': None}, True
+        image = {'image': 'repo', 'path': 'out_path', 'driver': 'glance',
+                 'repo': 'test', 'tag': 'testtag'}
+        mock_pull.return_value = image, True
         mock_create.side_effect = exception.DockerError(
             message="Docker Error occurred")
+        self.compute_manager._resource_tracker = FakeResourceTracker()
+        networks = []
         self.compute_manager._do_container_run(self.context,
-                                               container)
+                                               container,
+                                               networks)
         mock_save.assert_called_with(self.context)
-        mock_fail.assert_called_with(self.context,
-                                     container, 'Docker Error occurred')
+        mock_fail.assert_called_with(
+            self.context, container, 'Docker Error occurred', unset_host=True)
         mock_pull.assert_any_call(self.context, container.image, 'latest',
                                   'always', 'glance')
-        mock_create.assert_called_once_with(self.context, container, None,
-                                            {'name': 'nginx', 'path': None})
+        mock_create.assert_called_once_with(
+            self.context, container, image, networks)
 
+    @mock.patch.object(compute_node_tracker.ComputeNodeTracker,
+                       'remove_usage_from_container')
+    @mock.patch.object(Container, 'destroy')
     @mock.patch.object(Container, 'save')
     @mock.patch.object(fake_driver, 'delete')
-    def test_container_delete(self, mock_delete, mock_save):
+    def test_container_delete(self, mock_delete, mock_save, mock_cnt_destroy,
+                              mock_remove_usage):
         container = Container(self.context, **utils.get_test_container())
         self.compute_manager.container_delete(self. context, container, False)
         mock_save.assert_called_with(self.context)
-        mock_delete.assert_called_once_with(container, False)
+        mock_delete.assert_called_once_with(self.context, container, False)
+        mock_cnt_destroy.assert_called_once_with(self.context)
+        mock_remove_usage.assert_called_once_with(self.context, container,
+                                                  True)
 
     @mock.patch.object(manager.Manager, '_fail_container')
     @mock.patch.object(Container, 'save')
@@ -232,14 +272,14 @@ class TestManager(base.TestCase):
 
     @mock.patch.object(manager.Manager, '_fail_container')
     @mock.patch.object(fake_driver, 'delete_sandbox')
-    @mock.patch.object(fake_driver, 'get_sandbox_id')
     @mock.patch.object(Container, 'save')
     @mock.patch.object(fake_driver, 'delete')
     def test_container_delete_sandbox_failed(self, mock_delete, mock_save,
-                                             mock_sandbox, mock_delete_sandbox,
+                                             mock_delete_sandbox,
                                              mock_fail):
+        self.compute_manager.use_sandbox = True
         container = Container(self.context, **utils.get_test_container())
-        mock_sandbox.return_value = "sandbox_id"
+        container.set_sandbox_id("sandbox_id")
         mock_delete_sandbox.side_effect = exception.ZunException(
             message="Unexpected exception")
         self.assertRaises(exception.ZunException,
@@ -252,7 +292,7 @@ class TestManager(base.TestCase):
     @mock.patch.object(fake_driver, 'list')
     def test_container_list(self, mock_list):
         self.compute_manager.container_list(self.context)
-        mock_list.assert_called_once_with()
+        mock_list.assert_called_once_with(self.context)
 
     @mock.patch.object(fake_driver, 'list')
     def test_container_list_failed(self, mock_list):
@@ -265,7 +305,7 @@ class TestManager(base.TestCase):
     def test_container_show(self, mock_show):
         container = Container(self.context, **utils.get_test_container())
         self.compute_manager.container_show(self.context, container)
-        mock_show.assert_called_once_with(container)
+        mock_show.assert_called_once_with(self.context, container)
 
     @mock.patch.object(fake_driver, 'show')
     def test_container_show_failed(self, mock_show):
@@ -281,7 +321,7 @@ class TestManager(base.TestCase):
         container = Container(self.context, **utils.get_test_container())
         self.compute_manager._do_container_reboot(self.context, container, 10)
         mock_save.assert_called_with(self.context)
-        mock_reboot.assert_called_once_with(container, 10)
+        mock_reboot.assert_called_once_with(self.context, container, 10)
 
     @mock.patch.object(manager.Manager, '_fail_container')
     @mock.patch.object(Container, 'save')
@@ -304,7 +344,7 @@ class TestManager(base.TestCase):
         container = Container(self.context, **utils.get_test_container())
         self.compute_manager._do_container_stop(self.context, container, 10)
         mock_save.assert_called_with(self.context)
-        mock_stop.assert_called_once_with(container, 10)
+        mock_stop.assert_called_once_with(self.context, container, 10)
 
     @mock.patch.object(manager.Manager, '_fail_container')
     @mock.patch.object(Container, 'save')
@@ -326,7 +366,7 @@ class TestManager(base.TestCase):
         container = Container(self.context, **utils.get_test_container())
         self.compute_manager._do_container_start(self.context, container)
         mock_save.assert_called_with(self.context)
-        mock_start.assert_called_once_with(container)
+        mock_start.assert_called_once_with(self.context, container)
 
     @mock.patch.object(Container, 'save')
     @mock.patch.object(manager.Manager, '_fail_container')
@@ -347,7 +387,7 @@ class TestManager(base.TestCase):
     def test_container_pause(self, mock_pause):
         container = Container(self.context, **utils.get_test_container())
         self.compute_manager._do_container_pause(self.context, container)
-        mock_pause.assert_called_once_with(container)
+        mock_pause.assert_called_once_with(self.context, container)
 
     @mock.patch.object(manager.Manager, '_fail_container')
     @mock.patch.object(fake_driver, 'pause')
@@ -365,7 +405,7 @@ class TestManager(base.TestCase):
     def test_container_unpause(self, mock_unpause):
         container = Container(self.context, **utils.get_test_container())
         self.compute_manager._do_container_unpause(self.context, container)
-        mock_unpause.assert_called_once_with(container)
+        mock_unpause.assert_called_once_with(self.context, container)
 
     @mock.patch.object(manager.Manager, '_fail_container')
     @mock.patch.object(fake_driver, 'unpause')
@@ -385,9 +425,9 @@ class TestManager(base.TestCase):
         self.compute_manager.container_logs(self.context,
                                             container, True, True,
                                             False, 'all', None)
-        mock_logs.assert_called_once_with(container, stderr=True, stdout=True,
-                                          timestamps=False, tail='all',
-                                          since=None)
+        mock_logs.assert_called_once_with(
+            self.context, container, stderr=True, stdout=True,
+            timestamps=False, tail='all', since=None)
 
     @mock.patch.object(fake_driver, 'show_logs')
     def test_container_logs_failed(self, mock_logs):
@@ -405,8 +445,8 @@ class TestManager(base.TestCase):
         container = Container(self.context, **utils.get_test_container())
         self.compute_manager.container_exec(
             self.context, container, 'fake_cmd', True, False)
-        mock_execute_create.assert_called_once_with(container, 'fake_cmd',
-                                                    False)
+        mock_execute_create.assert_called_once_with(
+            self.context, container, 'fake_cmd', False)
         mock_execute_run.assert_called_once_with('fake_exec_id', 'fake_cmd')
 
     @mock.patch.object(fake_driver, 'execute_create')
@@ -421,7 +461,7 @@ class TestManager(base.TestCase):
     def test_container_kill(self, mock_kill):
         container = Container(self.context, **utils.get_test_container())
         self.compute_manager._do_container_kill(self.context, container, None)
-        mock_kill.assert_called_once_with(container, None)
+        mock_kill.assert_called_once_with(self.context, container, None)
 
     @mock.patch.object(manager.Manager, '_fail_container')
     @mock.patch.object(fake_driver, 'kill')
@@ -442,7 +482,7 @@ class TestManager(base.TestCase):
         self.compute_manager.container_update(self.context, container,
                                               {'memory': 512})
         mock_save.assert_called_with(self.context)
-        mock_update.assert_called_once_with(container)
+        mock_update.assert_called_once_with(self.context, container)
 
     @mock.patch.object(fake_driver, 'update')
     def test_container_update_failed(self, mock_update):
@@ -452,18 +492,20 @@ class TestManager(base.TestCase):
                           self.compute_manager.container_update,
                           self.context, container, {})
 
-    @mock.patch.object(fake_driver, 'attach')
-    @mock.patch('zun.container.driver.ContainerDriver.get_websocket_url')
-    def test_container_attach(self, mock_attach, mock_getwebsocket_url):
+    @mock.patch.object(fake_driver, 'get_websocket_url')
+    @mock.patch.object(Container, 'save')
+    def test_container_attach_successful(self, mock_save,
+                                         mock_get_websocket_url):
         container = Container(self.context, **utils.get_test_container())
-        mock_getwebsocket_url.return_value = "ws://test"
+        mock_get_websocket_url.return_value = "ws://test"
         self.compute_manager.container_attach(self.context, container)
-        mock_attach.assert_called_once_with(container)
+        mock_get_websocket_url.assert_called_once_with(self.context, container)
+        mock_save.assert_called_once_with(self.context)
 
-    @mock.patch.object(fake_driver, 'attach')
-    def test_container_attach_failed(self, mock_attach):
+    @mock.patch.object(fake_driver, 'get_websocket_url')
+    def test_container_attach_failed(self, mock_get_websocket_url):
         container = Container(self.context, **utils.get_test_container())
-        mock_attach.side_effect = Exception
+        mock_get_websocket_url.side_effect = Exception
         self.assertRaises(exception.ZunException,
                           self.compute_manager.container_attach,
                           self.context, container)
@@ -471,9 +513,10 @@ class TestManager(base.TestCase):
     @mock.patch.object(fake_driver, 'resize')
     def test_container_resize(self, mock_resize):
         container = Container(self.context, **utils.get_test_container())
-        self.compute_manager.container_resize(self.context,
-                                              container, "100", "100")
-        mock_resize.assert_called_once_with(container, "100", "100")
+        self.compute_manager.container_resize(
+            self.context, container, "100", "100")
+        mock_resize.assert_called_once_with(
+            self.context, container, "100", "100")
 
     @mock.patch.object(fake_driver, 'resize')
     def test_container_resize_failed(self, mock_resize):
@@ -511,7 +554,7 @@ class TestManager(base.TestCase):
         mock_pull.assert_any_call(self.context, image.repo, image.tag)
         mock_save.assert_called_once()
         mock_inspect.assert_called_once_with(repo_tag)
-        mock_load.assert_called_once_with(repo_tag, ret['path'])
+        mock_load.assert_called_once_with(ret['path'])
 
     @mock.patch.object(fake_driver, 'execute_resize')
     def test_container_exec_resize(self, mock_resize):
@@ -525,3 +568,28 @@ class TestManager(base.TestCase):
         self.assertRaises(exception.DockerError,
                           self.compute_manager.container_exec_resize,
                           self.context, 'fake_exec_id', "100", "100")
+
+    @mock.patch('zun.image.driver.upload_image_data')
+    @mock.patch.object(fake_driver, 'get_image')
+    @mock.patch.object(fake_driver, 'commit')
+    def test_container_commit(self, mock_commit,
+                              mock_get_image, mock_upload_image_data):
+        container = Container(self.context, **utils.get_test_container())
+        mock_get_image_response = mock.MagicMock()
+        mock_get_image_response.data = StringIO().read()
+        mock_get_image.return_value = mock_get_image_response
+        mock_upload_image_data.return_value = mock.MagicMock()
+
+        self.compute_manager._do_container_commit(self.context,
+                                                  mock_get_image_response,
+                                                  container, 'repo', 'tag')
+        mock_commit.assert_called_once_with(
+            self.context, container, 'repo', 'tag')
+
+    @mock.patch.object(fake_driver, 'commit')
+    def test_container_commit_failed(self, mock_commit):
+        container = Container(self.context, **utils.get_test_container())
+        mock_commit.side_effect = exception.DockerError
+        self.assertRaises(exception.DockerError,
+                          self.compute_manager._do_container_commit,
+                          self.context, container, 'repo', 'tag')
