@@ -61,12 +61,16 @@ class TestDockerDriver(base.DriverTestCase):
         self.driver.inspect_image(mock_image)
         self.mock_docker.inspect_image.assert_called_once_with(mock_image)
 
+    def test_get_image(self):
+        self.mock_docker.get_image = mock.Mock()
+        self.driver.get_image(name='image_name')
+        self.mock_docker.get_image.assert_called_once_with('image_name')
+
     def test_load_image(self):
         self.mock_docker.load_image = mock.Mock()
         mock_open_file = mock.mock_open()
         with mock.patch('zun.container.docker.driver.open', mock_open_file):
-            mock_image = mock.MagicMock()
-            self.driver.load_image(mock_image, 'test')
+            self.driver.load_image('test')
             self.mock_docker.load_image.assert_called_once_with(
                 mock_open_file.return_value)
 
@@ -75,20 +79,22 @@ class TestDockerDriver(base.DriverTestCase):
         self.driver.images(repo='test')
         self.mock_docker.images.assert_called_once_with('test', False)
 
+    @mock.patch('zun.network.kuryr_network.KuryrNetwork'
+                '.connect_container_to_network')
+    @mock.patch('zun.common.utils.get_security_group_ids')
     @mock.patch('zun.objects.container.Container.save')
-    def test_create_image_path_is_none(self, mock_save):
+    def test_create_image_path_is_none(self, mock_save,
+                                       mock_get_security_group_ids,
+                                       mock_connect):
         self.mock_docker.create_host_config = mock.Mock(
             return_value={'Id1': 'val1', 'key2': 'val2'})
         self.mock_docker.create_container = mock.Mock(
             return_value={'Id': 'val1', 'key1': 'val2'})
-        image = {'path': ''}
+        image = {'path': '', 'image': '', 'repo': '', 'tag': ''}
         mock_container = self.mock_default_container
         result_container = self.driver.create(self.context, mock_container,
-                                              'test_sandbox', image)
+                                              image, [])
         host_config = {}
-        host_config['network_mode'] = 'container:test_sandbox'
-        host_config['ipc_mode'] = 'container:test_sandbox'
-        host_config['volumes_from'] = 'test_sandbox'
         host_config['mem_limit'] = '512m'
         host_config['cpu_quota'] = 100000
         host_config['cpu_period'] = 100000
@@ -107,7 +113,7 @@ class TestDockerDriver(base.DriverTestCase):
             'tty': True,
         }
         self.mock_docker.create_container.assert_called_once_with(
-            mock_container.image, **kwargs)
+            image['repo'] + ":" + image['tag'], **kwargs)
         self.assertEqual('val1', result_container.container_id)
         self.assertEqual(result_container.status,
                          consts.CREATED)
@@ -115,7 +121,7 @@ class TestDockerDriver(base.DriverTestCase):
     def test_delete_success(self):
         self.mock_docker.remove_container = mock.Mock()
         mock_container = mock.MagicMock()
-        self.driver.delete(mock_container, True)
+        self.driver.delete(self.context, mock_container, True)
         self.mock_docker.remove_container.assert_called_once_with(
             mock_container.container_id, force=True)
 
@@ -125,7 +131,7 @@ class TestDockerDriver(base.DriverTestCase):
             self.mock_docker.remove_container = mock.Mock(
                 side_effect=errors.APIError('Error', '', ''))
             mock_container = mock.MagicMock()
-            self.driver.delete(mock_container, True)
+            self.driver.delete(self.context, mock_container, True)
             self.mock_docker.remove_container.assert_called_once_with(
                 mock_container.container_id, force=True)
             self.assertEqual(1, mock_init.call_count)
@@ -137,7 +143,7 @@ class TestDockerDriver(base.DriverTestCase):
                 side_effect=errors.APIError('Error', '', ''))
             mock_container = mock.MagicMock()
             self.assertRaises(errors.APIError, self.driver.delete,
-                              mock_container,
+                              self.context, mock_container,
                               True)
             self.mock_docker.remove_container.assert_called_once_with(
                 mock_container.container_id, force=True)
@@ -168,14 +174,24 @@ class TestDockerDriver(base.DriverTestCase):
         self.mock_docker.inspect_container = mock.Mock(
             return_value={'State': 'running'})
         mock_container = mock.MagicMock()
-        self.driver.show(mock_container)
+        self.driver.show(self.context, mock_container)
         self.mock_docker.inspect_container.assert_called_once_with(
             mock_container.container_id)
 
+    def test_show_command(self):
+        self.mock_docker.inspect_container = mock.Mock(
+            return_value={'State': 'running',
+                          'Config': {'Cmd': ['fake_command']}})
+        mock_container = mock.MagicMock()
+        self.driver.show(self.context, mock_container)
+        self.mock_docker.inspect_container.assert_called_once_with(
+            mock_container.container_id)
+        self.assertEqual('fake_command', mock_container.command)
+
     def test_show_fail_container_id_is_none(self):
         mock_container = mock.MagicMock()
         mock_container.container_id = None
-        result_container = self.driver.show(mock_container)
+        result_container = self.driver.show(self.context, mock_container)
         self.assertIsNone(result_container.container_id)
 
     def test_show_fail_container_status_error(self):
@@ -183,8 +199,8 @@ class TestDockerDriver(base.DriverTestCase):
                                return_value='404 Not Found') as mock_init:
             self.mock_docker.inspect_container = mock.Mock(
                 side_effect=errors.APIError('Error', '', ''))
-            mock_container = mock.MagicMock()
-            result_container = self.driver.show(mock_container)
+            mock_container = mock.MagicMock(auto_remove=False)
+            result_container = self.driver.show(self.context, mock_container)
             self.mock_docker.inspect_container.assert_called_once_with(
                 mock_container.container_id)
             self.assertEqual(result_container.status,
@@ -198,7 +214,7 @@ class TestDockerDriver(base.DriverTestCase):
                 side_effect=errors.APIError('Error', '', ''))
             mock_container = mock.MagicMock()
             self.assertRaises(errors.APIError, self.driver.show,
-                              mock_container)
+                              self.context, mock_container)
             self.mock_docker.inspect_container.assert_called_once_with(
                 mock_container.container_id)
             self.assertEqual(1, mock_init.call_count)
@@ -206,7 +222,8 @@ class TestDockerDriver(base.DriverTestCase):
     def test_reboot(self):
         self.mock_docker.restart = mock.Mock()
         mock_container = mock.MagicMock()
-        result_container = self.driver.reboot(mock_container, '30')
+        result_container = self.driver.reboot(
+            self.context, mock_container, '30')
         self.mock_docker.restart.assert_called_once_with(
             mock_container.container_id, timeout=30)
         self.assertEqual(result_container.status,
@@ -215,7 +232,7 @@ class TestDockerDriver(base.DriverTestCase):
     def test_stop(self):
         self.mock_docker.stop = mock.Mock()
         mock_container = mock.MagicMock()
-        result_container = self.driver.stop(mock_container, '30')
+        result_container = self.driver.stop(self.context, mock_container, '30')
         self.mock_docker.stop.assert_called_once_with(
             mock_container.container_id,
             timeout=30)
@@ -225,7 +242,7 @@ class TestDockerDriver(base.DriverTestCase):
     def test_start(self):
         self.mock_docker.start = mock.Mock()
         mock_container = mock.MagicMock()
-        result_container = self.driver.start(mock_container)
+        result_container = self.driver.start(self.context, mock_container)
         self.mock_docker.start.assert_called_once_with(
             mock_container.container_id)
         self.assertEqual(result_container.status,
@@ -234,7 +251,7 @@ class TestDockerDriver(base.DriverTestCase):
     def test_pause(self):
         self.mock_docker.pause = mock.Mock()
         mock_container = mock.MagicMock()
-        result_container = self.driver.pause(mock_container)
+        result_container = self.driver.pause(self.context, mock_container)
         self.mock_docker.pause.assert_called_once_with(
             mock_container.container_id)
         self.assertEqual(result_container.status,
@@ -243,24 +260,25 @@ class TestDockerDriver(base.DriverTestCase):
     def test_unpause(self):
         self.mock_docker.unpause = mock.Mock()
         mock_container = mock.MagicMock()
-        result_container = self.driver.unpause(mock_container)
+        result_container = self.driver.unpause(self.context, mock_container)
         self.mock_docker.unpause.assert_called_once_with(
             mock_container.container_id)
         self.assertEqual(result_container.status,
                          consts.RUNNING)
 
     def test_show_logs(self):
-        self.mock_docker.get_container_logs = mock.Mock()
+        self.mock_docker.logs = mock.Mock()
         mock_container = mock.MagicMock()
-        self.driver.show_logs(mock_container)
-        self.mock_docker.get_container_logs.assert_called_once_with(
+        self.driver.show_logs(self.context, mock_container)
+        self.mock_docker.logs.assert_called_once_with(
             mock_container.container_id, True, True, False, False,
             'all', None)
 
     def test_execute_create(self):
         self.mock_docker.exec_create = mock.Mock(return_value={'Id': 'test'})
         mock_container = mock.MagicMock()
-        exec_id = self.driver.execute_create(mock_container, 'ls')
+        exec_id = self.driver.execute_create(
+            self.context, mock_container, 'ls')
         self.assertEqual('test', exec_id)
         self.mock_docker.exec_create.assert_called_once_with(
             mock_container.container_id, 'ls', stdin=False, tty=False)
@@ -268,7 +286,7 @@ class TestDockerDriver(base.DriverTestCase):
     def test_execute_run(self):
         self.mock_docker.exec_start = mock.Mock(return_value='test')
         self.mock_docker.exec_inspect = mock.Mock(
-            return_value={u'ExitCode': 0})
+            return_value={'ExitCode': 0})
         self.driver.execute_run('test', 'ls')
         self.mock_docker.exec_start.assert_called_once_with('test', False,
                                                             False, False)
@@ -276,79 +294,48 @@ class TestDockerDriver(base.DriverTestCase):
 
     def test_kill_successful_signal_is_none(self):
         self.mock_docker.kill = mock.Mock()
-        self.mock_docker.inspect_container = mock.Mock(
-            return_value={'State': 'exited'})
         mock_container = mock.MagicMock()
-        self.driver.kill(mock_container, signal=None)
+        self.driver.kill(self.context, mock_container, signal=None)
         self.mock_docker.kill.assert_called_once_with(
             mock_container.container_id)
-        self.mock_docker.inspect_container.assert_called_once_with(
-            mock_container.container_id)
 
     def test_kill_successful_signal_is_not_none(self):
         self.mock_docker.kill = mock.Mock()
-        self.mock_docker.inspect_container = mock.Mock(
-            return_value={'State': 'exited'})
         mock_container = mock.MagicMock()
-        self.driver.kill(mock_container, signal='test')
+        self.driver.kill(self.context, mock_container, signal='test')
         self.mock_docker.kill.assert_called_once_with(
             mock_container.container_id,
             'test')
-        self.mock_docker.inspect_container.assert_called_once_with(
-            mock_container.container_id)
-
-    def test_kill_fail_container_status_error(self):
-        with mock.patch.object(errors.APIError, '__str__',
-                               return_value='404 Not Found') as mock_init:
-            self.mock_docker.kill = mock.Mock()
-            self.mock_docker.inspect_container = mock.Mock(
-                side_effect=errors.APIError('Error', '', ''))
-            mock_container = mock.MagicMock()
-            result_container = self.driver.kill(mock_container,
-                                                signal='test')
-            self.mock_docker.kill.assert_called_once_with(
-                mock_container.container_id, 'test')
-            self.mock_docker.inspect_container.assert_called_once_with(
-                mock_container.container_id)
-            self.assertEqual(result_container.status,
-                             consts.ERROR)
-            self.assertEqual(2, mock_init.call_count)
-
-    def test_kill_fail_api_error(self):
-        with mock.patch.object(errors.APIError, '__str__',
-                               return_value='test') as mock_init:
-            self.mock_docker.kill = mock.Mock()
-            self.mock_docker.inspect_container = mock.Mock(
-                side_effect=errors.APIError('Error', '', ''))
-            mock_container = mock.MagicMock()
-            self.assertRaises(errors.APIError, self.driver.kill,
-                              mock_container,
-                              'test')
-            self.mock_docker.kill.assert_called_once_with(
-                mock_container.container_id, 'test')
-            self.mock_docker.inspect_container.assert_called_once_with(
-                mock_container.container_id)
-            self.assertEqual(1, mock_init.call_count)
 
     def test_resize(self):
         self.mock_docker.resize = mock.Mock()
         mock_container = mock.MagicMock()
-        self.driver.resize(mock_container, "100", "100")
+        self.driver.resize(self.context, mock_container, "100", "100")
         self.mock_docker.resize.assert_called_once_with(
             mock_container.container_id, 100, 100)
 
+    def test_commit(self):
+        self.mock_docker.commit = mock.Mock()
+        mock_container = mock.MagicMock()
+        self.driver.commit(self.context, mock_container, "repo", "tag")
+        self.mock_docker.commit.assert_called_once_with(
+            mock_container.container_id, "repo", "tag")
+
     @mock.patch('zun.network.kuryr_network.KuryrNetwork'
                 '.connect_container_to_network')
     @mock.patch('zun.container.docker.driver.DockerDriver.get_sandbox_name')
-    def test_create_sandbox(self, mock_get_sandbox_name, mock_connect):
+    @mock.patch('zun.common.utils.get_security_group_ids')
+    def test_create_sandbox(self, mock_get_security_group_ids,
+                            mock_get_sandbox_name, mock_connect):
         sandbox_name = 'my_test_sandbox'
         mock_get_sandbox_name.return_value = sandbox_name
         self.mock_docker.create_container = mock.Mock(
             return_value={'Id': 'val1', 'key1': 'val2'})
         mock_container = mock.MagicMock()
-        with mock.patch.object(self.driver, '_get_available_network'):
-            result_sandbox_id = self.driver.create_sandbox(
-                self.context, mock_container, 'kubernetes/pause')
+        requested_networks = []
+        result_sandbox_id = self.driver.create_sandbox(
+            self.context, mock_container, requested_networks,
+            'kubernetes/pause')
         self.mock_docker.create_container.assert_called_once_with(
             'kubernetes/pause', name=sandbox_name, hostname=sandbox_name)
         self.assertEqual(result_sandbox_id, 'val1')
@@ -356,24 +343,28 @@ class TestDockerDriver(base.DriverTestCase):
     @mock.patch('zun.network.kuryr_network.KuryrNetwork'
                 '.connect_container_to_network')
     @mock.patch('zun.container.docker.driver.DockerDriver.get_sandbox_name')
-    def test_create_sandbox_with_long_name(self, mock_get_sandbox_name,
+    @mock.patch('zun.common.utils.get_security_group_ids')
+    def test_create_sandbox_with_long_name(self, mock_get_security_group_ids,
+                                           mock_get_sandbox_name,
                                            mock_connect):
         sandbox_name = 'x' * 100
         mock_get_sandbox_name.return_value = sandbox_name
         self.mock_docker.create_container = mock.Mock(
             return_value={'Id': 'val1', 'key1': 'val2'})
         mock_container = mock.MagicMock()
-        with mock.patch.object(self.driver, '_get_available_network'):
-            result_sandbox_id = self.driver.create_sandbox(
-                self.context, mock_container, 'kubernetes/pause')
+        requested_networks = []
+        result_sandbox_id = self.driver.create_sandbox(
+            self.context, mock_container, requested_networks,
+            'kubernetes/pause')
         self.mock_docker.create_container.assert_called_once_with(
             'kubernetes/pause', name=sandbox_name, hostname=sandbox_name[:63])
         self.assertEqual(result_sandbox_id, 'val1')
 
     def test_delete_sandbox(self):
         self.mock_docker.remove_container = mock.Mock()
-        self.driver.delete_sandbox(context=self.context,
-                                   sandbox_id='test_sandbox_id')
+        mock_container = mock.MagicMock()
+        mock_container.get_sandbox_id.return_value = 'test_sandbox_id'
+        self.driver.delete_sandbox(self.context, mock_container)
         self.mock_docker.remove_container.assert_called_once_with(
             'test_sandbox_id', force=True)
 
@@ -383,24 +374,6 @@ class TestDockerDriver(base.DriverTestCase):
                                  sandbox_id='test_sandbox_id')
         self.mock_docker.stop.assert_called_once_with('test_sandbox_id')
 
-    def test_get_sandbox_none_id(self):
-        mock_container = mock.MagicMock()
-        mock_container.meta = None
-        result_sandbox_id = self.driver.get_sandbox_id(mock_container)
-        self.assertIsNone(result_sandbox_id)
-
-    def test_get_sandbox_not_none_id(self):
-        mock_container = mock.MagicMock()
-        result_sandbox_id = self.driver.get_sandbox_id(mock_container)
-        self.assertEqual(result_sandbox_id,
-                         mock_container.meta.get('sandbox_id', None))
-
-    def test_set_sandbox_id(self):
-        mock_container = mock.MagicMock(meta={'sandbox_id': 'test_sandbox_id'})
-        self.driver.set_sandbox_id(mock_container, 'test_sandbox_id')
-        self.assertEqual(mock_container.meta['sandbox_id'],
-                         'test_sandbox_id')
-
     def test_get_sandbox_name(self):
         mock_container = mock.MagicMock(
             uuid='ea8e2a25-2901-438d-8157-de7ffd68d051')
@@ -416,32 +389,6 @@ class TestDockerDriver(base.DriverTestCase):
         self.assertEqual(result_container_name,
                          'zun-ea8e2a25-2901-438d-8157-de7ffd68d051')
 
-    @mock.patch('zun.common.clients.OpenStackClients.neutron')
-    @mock.patch('zun.container.docker.driver.DockerDriver.get_sandbox_id')
-    def test_get_addresses(self, mock_get_sandbox_id, mock_neutron_client):
-        mock_port_id = 'f17d93cf-28e1-4a85-957d-8de8c3f91dfe'
-        mock_net_name = 'test_net'
-        mock_list_ports = mock_neutron_client.return_value.list_ports
-        mock_list_ports.return_value = {'ports': [{'id': mock_port_id}]}
-        mock_show_network = mock_neutron_client.return_value.show_network
-        mock_show_network.return_value = {'network': {'name': mock_net_name}}
-        mock_get_sandbox_id.return_value = 'test_sandbox_id'
-        self.mock_docker.inspect_container = mock.Mock(
-            return_value={'NetworkSettings': {'Networks': {'default': {
-                'IPAddress': '127.0.0.1',
-                'GlobalIPv6Address': 'fe80::4',
-                'MacAddress': 'fa:16:3e:85:e7:d5',
-            }}}})
-        mock_container = mock.MagicMock()
-        result_addresses = self.driver.get_addresses(self.context,
-                                                     mock_container)
-        self.mock_docker.inspect_container.assert_called_once_with(
-            'test_sandbox_id')
-        expected_addresses = {mock_net_name: [
-            {'addr': '127.0.0.1', 'version': 4, 'port': mock_port_id},
-            {'addr': 'fe80::4', 'version': 6, 'port': mock_port_id}]}
-        self.assertEqual(expected_addresses, result_addresses)
-
     def test_execute_resize(self):
         self.mock_docker.exec_resize = mock.Mock()
         fake_exec_id = 'fake_id'
@@ -459,9 +406,10 @@ class TestDockerDriver(base.DriverTestCase):
                                               'Architecture': 'x86_64',
                                               'OSType': 'linux',
                                               'OperatingSystem': 'CentOS',
-                                              'KernelVersion': '3.10.0-123'}
+                                              'KernelVersion': '3.10.0-123',
+                                              'Labels': ['dev.type=product']}
         (total, running, paused, stopped, cpus, architecture,
-         os_type, os, kernel_version) = self.driver.get_host_info()
+         os_type, os, kernel_version, labels) = self.driver.get_host_info()
         self.assertEqual(10, total)
         self.assertEqual(8, running)
         self.assertEqual(0, paused)
@@ -471,6 +419,7 @@ class TestDockerDriver(base.DriverTestCase):
         self.assertEqual('linux', os_type)
         self.assertEqual('CentOS', os)
         self.assertEqual('3.10.0-123', kernel_version)
+        self.assertEqual({"dev.type": "product"}, labels)
 
     def test_get_cpu_used(self):
         self.mock_docker.containers = mock.Mock()
@@ -483,6 +432,38 @@ class TestDockerDriver(base.DriverTestCase):
         cpu_used = self.driver.get_cpu_used()
         self.assertEqual(1.0, cpu_used)
 
+    def test_stats(self):
+        self.mock_docker.stats = mock.Mock()
+        mock_container = mock.MagicMock()
+        self.mock_docker.stats.return_value = {
+            'cpu_stats': {'cpu_usage': {'usage_in_usermode': 1000000000,
+                                        'total_usage': 1000000000},
+                          'system_cpu_usage': 1000000000000},
+            'blkio_stats': {'io_service_bytes_recursive':
+                            [{'major': 253, 'value': 10000000,
+                              'minor': 4, 'op': 'Read'},
+                             {'major': 253, 'value': 0,
+                              'minor': 4, 'op': 'Write'},
+                             {'major': 253, 'value': 0,
+                              'minor': 4, 'op': 'Sync'},
+                             {'major': 253, 'value': 10000000,
+                              'minor': 4, 'op': 'Async'},
+                             {'major': 253, 'value': 10000000,
+                              'minor': 4, 'op': 'Total'}]},
+            'memory_stats': {'usage': 104857600,
+                             'limit': 1048576000},
+            'networks': {'eth0':
+                            {'tx_dropped': 0, 'rx_packets': 2, 'rx_bytes': 200,
+                             'tx_errors': 0, 'rx_errors': 0, 'tx_bytes': 200,
+                             'rx_dropped': 0, 'tx_packets': 2}}}
+        stats_info = self.driver.stats(self.context, mock_container)
+        self.assertEqual(0.1, stats_info['CPU %'])
+        self.assertEqual(100, stats_info['MEM USAGE(MiB)'])
+        self.assertEqual(1000, stats_info['MEM LIMIT(MiB)'])
+        self.assertEqual(10, stats_info['MEM %'])
+        self.assertEqual('10000000/0', stats_info['BLOCK I/O(B)'])
+        self.assertEqual('200/200', stats_info['NET I/O(B)'])
+
 
 class TestNovaDockerDriver(base.DriverTestCase):
     def setUp(self):
@@ -557,19 +538,17 @@ class TestNovaDockerDriver(base.DriverTestCase):
 
     @mock.patch('zun.container.docker.driver.'
                 'NovaDockerDriver._find_server_by_container_id')
-    @mock.patch('zun.container.docker.driver.NovaDockerDriver.get_sandbox_id')
     @mock.patch('zun.common.nova.NovaClient')
-    def test_get_addresses(self, mock_nova_client, mock_get_sandbox_id,
+    def test_get_addresses(self, mock_nova_client,
                            mock_find_server_by_container_id):
         nova_client_instance = mock.MagicMock()
         nova_client_instance.get_addresses.return_value = 'test_address'
         mock_nova_client.return_value = nova_client_instance
-        mock_get_sandbox_id.return_value = 'test_sanbox_id'
         mock_find_server_by_container_id.return_value = 'test_test_server_name'
         mock_container = mock.MagicMock()
+        mock_container.get_sandbox_id.return_value = 'test_sanbox_id'
         result_address = self.driver.get_addresses(self.context,
                                                    mock_container)
-        mock_get_sandbox_id.assert_called_once_with(mock_container)
         mock_find_server_by_container_id.assert_called_once_with(
             'test_sanbox_id')
         nova_client_instance.get_addresses.assert_called_once_with(
@@ -587,9 +566,11 @@ class TestNovaDockerDriver(base.DriverTestCase):
         self.driver = DockerDriver()
         mock_output.return_value = LSCPU_ON
         conf.CONF.set_override('floating_cpu_set', "0")
-        mock_mem.return_value = (100 * units.Ki, 50 * units.Ki, 50 * units.Ki)
+        mock_mem.return_value = (100 * units.Ki, 50 * units.Ki, 50 * units.Ki,
+                                 50 * units.Ki)
         mock_info.return_value = (10, 8, 0, 2, 48, 'x86_64', 'linux',
-                                  'CentOS', '3.10.0-123')
+                                  'CentOS', '3.10.0-123',
+                                  {'dev.type': 'product'})
         mock_cpu_used.return_value = 1.0
         node_obj = objects.ComputeNode()
         self.driver.get_available_resources(node_obj)
@@ -607,3 +588,4 @@ class TestNovaDockerDriver(base.DriverTestCase):
         self.assertEqual('linux', node_obj.os_type)
         self.assertEqual('CentOS', node_obj.os)
         self.assertEqual('3.10.0-123', node_obj.kernel_version)
+        self.assertEqual({'dev.type': 'product'}, node_obj.labels)
